import {
  ReactiveAdapter,
  ReactiveSubscribe,
  camelize,
  adaptReactive,
} from "@cfcs/core";
import { onMounted, onUnmounted, Ref, ref } from "@vue/composition-api";
import { VueReactiveResult } from "./types";

export function useReactive<
  Instance extends ReactiveSubscribe<Record<string, any>>,
  State extends Record<string, any> = {},
  Methods extends keyof Partial<Instance> = any,
  Data = any,
  Events extends Record<string, any> = {},
  >(reactiveProps: ReactiveAdapter<Instance, State, Methods, Data, Events>,
): VueReactiveResult<Instance, State, Methods, Events> {
  const adaptResult = adaptReactive(reactiveProps);
  const reactiveState = adaptResult.state();
  const names = Object.keys(reactiveState);
  const refs: Record<string, Ref<any>> = {};
  for (const name in reactiveState) {
    refs[name] = ref(reactiveState[name]);
  }
  const methods = adaptResult.methods();

  onMounted(() => {
    adaptResult.mounted();

    const inst = adaptResult.instance();

    names.forEach(name => {
      inst.subscribe(name as any, (value: any) => {
        refs[name].value = value;
      });
    });

    adaptResult.init();
  });

  onUnmounted(() => {
    adaptResult.destroy();
  });

  const reactiveEvents = (reactiveProps.events || []);
  const events = reactiveEvents.reduce((result, name) => {
    result[camelize(`on ${name as any}`)] = (callback: (...args: any[]) => void) => {
      onMounted(() => {
        adaptResult.on(name as any, callback as any);
      });
      onUnmounted(() => {
        adaptResult.off(name as any, callback as any);
      });
    };

    return result;
  }, {} as Record<string, any>);

  return {
    ...refs,
    ...methods,
    ...events,
  } as any;
}
